#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import logging.handlers
from logging.config import dictConfig
import signal
import mraa
from platform import system

from ConfigManager import ConfigManager
from Gs1Cmd import Gs1Cmd
from Gs1Controller import Gs1Controller

if system() == 'Windows':
    from Emulator.OperatingPanel import OperatingPanel

_logger = logging.getLogger(__name__)

LEVEL = logging.DEBUG


class GS1(object):
    def __init__(self):
        """!
        Konfiguration der Ofensteuerung
        Initialisierung der HW Treiber
        Initialisierung des Betriebsstunden-Zählers
        Initialisierung des Command-Line Interface
        Initialisierung/Start Datenlogging (LH 8.6.3)
        """
        _logger.debug("GS1 init")

        self._config_manager = ConfigManager(pathfile="./part_r/pathfile.gs1")
        self._config_parser = self._config_manager.get_parser()

        # load MRAA BASEBOARD JSON
        mraa_baseboard = self._config_parser.get('path', 'gs1-mraa-baseboard')
        mraa.initJsonPlatform(mraa_baseboard)

        self._gs1_controller = Gs1Controller(self._config_manager)
        self._gs1_cmd2 = Gs1Cmd(self._gs1_controller)

        if system() == 'Windows':
            self._operatingPanel = OperatingPanel(self._gs1_controller)
            self._gs1_controller.setOperatingPanel(self._operatingPanel)

    def start(self):
        """!
        Startet GS1Controller und Kommando-Zeilen Interface.
        """
        # start all the stuff here maybe config manager depended ...
        self._gs1_controller.start()
        _logger.debug("GS1 started")
        self._gs1_cmd2.cmdloop()
        if system() == 'Windows':
            self._operatingPanel.addMessage('GS1 started')

    def exit(self):
        """!
        Beendet GS1Controller und Kommando-Zeilen Interface.
        """
        # todo fix exit
        # Mit geänderter Einrückung des Signalhandlers in main() sollte dies behoben sein.
        if system() == 'Windows':
            self._operatingPanel.stopCommunication()
        self._gs1_controller.exit()
        _logger.debug("GS1 exit")


def init_logger(console=False):
    if console:
        logging_config = dict(
            version=1,
            disable_existing_loggers=False,
            formatters={
                'f': {'format': '%(asctime)s %(name)-12s %(levelname)-8s %(message)s'}
            },
            handlers={
                'h': {'class': 'logging.StreamHandler',
                      'formatter': 'f',
                      'level': LEVEL}
            },
            root={
                'handlers': ['h'],
                'level': LEVEL,
            },
        )

        dictConfig(logging_config)
        #_logger = logging.getLogger()
    else:
        handler = logging.handlers.RotatingFileHandler("/home/root/gs1/part_rw/log/gs1_debug.log", 'a', 5 * 1024 * 1024, 10)
        formatter = logging.Formatter(logging.BASIC_FORMAT)
        handler.setFormatter(formatter)
        root = logging.getLogger()
        root.setLevel(LEVEL)
        root.addHandler(handler)


def main():
    """!
    Aufruf der Initialisierung (beim anlegen des Objekts)
    Start der Steuerung
    """
    init_logger()

    gs1 = GS1()

    def signal_handler(signal_in, frame):
        """!
        Signalhandler zum Unterbrechen der Programmausführung mit Ctrl-C
        """
        gs1.exit()

    # Register signal handler to allow interrupt with CTRL-C
    # Bugfix: Fehler wenn Einrückung geändert wird.
    signal.signal(signal.SIGINT, signal_handler)

    _logger.debug("GS1 start debug")

    # Launch gs1
    gs1.start()

    # Wait for end of process signal
    # signal.pause()

    gs1.exit()

if __name__ == '__main__':
    main()
